@@ -39,7 +39,9 @@ STYLE = "style"
 ATTRIBUTE = "attribute"
 ALLOW_CHOICE_DUPLICATES = "allow_choice_duplicates"
 
-BIND = "bind"  # TODO: What should I do with the nested types? (readonly and relevant) # noqa
+BIND = (
+    "bind"  # TODO: What should I do with the nested types? (readonly and relevant) # noqa
+)
 MEDIA = "media"
 CONTROL = "control"
 APPEARANCE = "appearance"
@@ -60,26 +62,21 @@ CHOICES = "choices"
 LIST_NAME = "list name"
 CASCADING_SELECT = "cascading_select"
 TABLE_LIST = "table-list"  # hyphenated because it goes in appearance, and convention for appearance column is dashes # noqa
+FIELD_LIST = "field-list"
 
-# The following are the possible sheet names:
 SURVEY = "survey"
 SETTINGS = "settings"
 EXTERNAL_CHOICES = "external_choices"
-# These sheet names are for list sheets
-CHOICES_AND_COLUMNS = "choices and columns"
-CASCADING_CHOICES = "cascades"
 
 OSM = "osm"
 OSM_TYPE = "binary"
 
 NAMESPACES = "namespaces"
 
+# The following are the possible sheet names:
 SUPPORTED_SHEET_NAMES = [
     SURVEY,
-    CASCADING_CHOICES,
     CHOICES,
-    COLUMNS,
-    CHOICES_AND_COLUMNS,
     SETTINGS,
     EXTERNAL_CHOICES,
     OSM,
