@@ -4,36 +4,15 @@ pyxform utils module.
 """
 import codecs
 import copy
+import csv
 import json
 import os
 import re
+from json.decoder import JSONDecodeError
 from xml.dom.minidom import Element, Text, parseString
 
-import unicodecsv as csv
 import xlrd
 
-try:
-    from json.decoder import JSONDecodeError
-except ImportError:
-    # json module raises a ValueError exception when it encounters an invalid JSON
-    JSONDecodeError = ValueError
-
-try:
-    unicode("str")
-except NameError:
-    unicode = str
-    basestring = str
-    unichr = chr
-else:
-    try:
-        unicode = unicode
-        basestring = basestring
-        unichr = unichr
-    except NameError:  # special cases where unicode is defined in python3
-        unicode = str
-        basestring = str
-        unichr = chr
-
 SEP = "_"
 
 # http://www.w3.org/TR/REC-xml/
@@ -102,7 +81,7 @@ def node(*args, **kwargs):
     tag = args[0] if len(args) > 0 else kwargs["tag"]
     args = args[1:]
     result = DetachableElement(tag)
-    unicode_args = [u for u in args if type(u) == unicode]
+    unicode_args = [u for u in args if type(u) == str]
     assert len(unicode_args) <= 1
     parsed_string = False
 
@@ -129,7 +108,9 @@ def node(*args, **kwargs):
                 # Move node's children to the result Element
                 # discarding node's root
                 for child in parsed_node.childNodes:
-                    result.appendChild(copy.deepcopy(child))
+                    # No tests seem to involve moving elements with children.
+                    # Deep clone used anyway in case of unknown edge cases.
+                    result.appendChild(child.cloneNode(deep=True))
         else:
             result.setAttribute(k, v)
 
@@ -140,9 +121,9 @@ def node(*args, **kwargs):
     for n in args:
         if type(n) == int or type(n) == float or type(n) == bytes:
             text_node = PatchedText()
-            text_node.data = unicode(n)
+            text_node.data = str(n)
             result.appendChild(text_node)
-        elif type(n) is not unicode:
+        elif type(n) is not str:
             result.appendChild(n)
     return result
 
@@ -179,7 +160,7 @@ def sheet_to_csv(workbook_path, csv_path, sheet_name):
         return False
     if not sheet or sheet.nrows < 2:
         return False
-    with open(csv_path, "wb") as f:
+    with open(csv_path, "w", newline="") as f:
         writer = csv.writer(f, quoting=csv.QUOTE_ALL)
         mask = [v and len(v.strip()) > 0 for v in sheet.row_values(0)]
         for row_idx in range(sheet.nrows):
@@ -206,11 +187,7 @@ def has_external_choices(json_struct):
     """
     if isinstance(json_struct, dict):
         for k, v in json_struct.items():
-            if (
-                k == "type"
-                and isinstance(v, basestring)
-                and v.startswith("select one external")
-            ):
+            if k == "type" and isinstance(v, str) and v.startswith("select one external"):
                 return True
             elif has_external_choices(v):
                 return True
@@ -225,7 +202,6 @@ def get_languages_with_bad_tags(languages):
     """
     Returns languages with invalid or missing IANA subtags.
     """
-    iana_subtags = []
     with open(os.path.join(os.path.dirname(__file__), "iana_subtags.txt")) as f:
         iana_subtags = f.read().splitlines()
 
@@ -250,7 +226,7 @@ def default_is_dynamic(element_default, element_type=None):
     * Contains arithmetic operator, including 'div' and 'mod' (except '-' for 'date' type).
     * Contains brackets, parentheses or braces.
     """
-    if not isinstance(element_default, basestring):
+    if not isinstance(element_default, str):
         return False
 
     dynamic_markers = {
@@ -273,7 +249,8 @@ def default_is_dynamic(element_default, element_type=None):
     return any(s in element_default for s in dynamic_markers)
 
 
-# If the first or second choice label includes a reference, we must use itext. Check the first two choices in case first is something like "Other".
+# If the first or second choice label includes a reference, we must use itext.
+# Check the first two choices in case first is something like "Other".
 def has_dynamic_label(choice_list, multi_language):
     if not multi_language:
         for i in range(0, min(2, len(choice_list))):
@@ -284,3 +261,51 @@ def has_dynamic_label(choice_list, multi_language):
             ):
                 return True
     return False
+
+
+def levenshtein_distance(a: str, b: str) -> int:
+    """
+    Calculate Levenshtein distance between two strings.
+
+    A Python translation of the "iterative with two matrix rows" algorithm from
+    Wikipedia: https://en.wikipedia.org/wiki/Levenshtein_distance
+
+    :param a: The first string to compare.
+    :param b: The second string to compare.
+    :return:
+    """
+    m = len(a)
+    n = len(b)
+
+    # create two work vectors of integer distances
+    v1 = [0 for _ in range(0, n + 1)]
+
+    # initialize v0 (the previous row of distances)
+    # this row is A[0][i]: edit distance for an empty s
+    # the distance is just the number of characters to delete from t
+    v0 = [i for i in range(0, n + 1)]
+
+    for i in range(0, m):
+        # calculate v1 (current row distances) from the previous row v0
+
+        # first element of v1 is A[i+1][0]
+        #   edit distance is delete (i+1) chars from s to match empty t
+        v1[0] = i + 1
+
+        # use formula to fill in the rest of the row
+        for j in range(0, n):
+            # calculating costs for A[i+1][j+1]
+            deletion_cost = v0[j + 1] + 1
+            insertion_cost = v1[j] + 1
+            if a[i] == b[j]:
+                substitution_cost = v0[j]
+            else:
+                substitution_cost = v0[j] + 1
+
+            v1[j + 1] = min((deletion_cost, insertion_cost, substitution_cost))
+
+        # copy v1 (current row) to v0 (previous row) for next iteration
+        # since data in v1 is always invalidated, a swap without copy could be more efficient
+        v0 = copy.copy(v1)
+    # after the last swap, the results of v1 are now in v0
+    return v0[n]
