@@ -2,21 +2,29 @@
 """
 A Python script to convert excel files into JSON.
 """
-from __future__ import print_function, unicode_literals
-
 import codecs
 import json
 import os
 import re
 import sys
 from collections import Counter
+from typing import TYPE_CHECKING
 
 from pyxform import aliases, constants
 from pyxform.errors import PyXFormError
-from pyxform.utils import basestring, is_valid_xml_tag, unicode, default_is_dynamic
+from pyxform.utils import default_is_dynamic, is_valid_xml_tag, levenshtein_distance
 from pyxform.xls2json_backends import csv_to_dict, xls_to_dict
 
+if TYPE_CHECKING:
+    from typing import Any, Dict, KeysView, Optional
+
+
 SMART_QUOTES = {"\u2018": "'", "\u2019": "'", "\u201c": '"', "\u201d": '"'}
+_MSG_SUPPRESS_SPELLING = (
+    " If you do not mean to include a sheet, to suppress this message, "
+    "prefix the sheet name with an underscore. For example 'setting' "
+    "becomes '_setting'."
+)
 
 
 def print_pyobj_to_json(pyobj, path=None):
@@ -75,7 +83,7 @@ def replace_smart_quotes_in_dict(_d):
     for key, value in _d.items():
         _changed = False
         for smart_quote, dumb_quote in SMART_QUOTES.items():
-            if isinstance(value, basestring):
+            if isinstance(value, str):
                 if smart_quote in value:
                     value = value.replace(smart_quote, dumb_quote)
                     _changed = True
@@ -148,8 +156,8 @@ def dealias_types(dict_array):
     """
     for row in dict_array:
         found_type = row.get(constants.TYPE)
-        if found_type in aliases._type.keys():
-            row[constants.TYPE] = aliases._type[found_type]
+        if found_type in aliases._type_alias_map.keys():
+            row[constants.TYPE] = aliases._type_alias_map[found_type]
     return dict_array
 
 
@@ -161,7 +169,7 @@ def clean_text_values(dict_array):
     for row in dict_array:
         replace_smart_quotes_in_dict(row)
         for key, value in row.items():
-            if isinstance(value, basestring):
+            if isinstance(value, str):
                 row[key] = re.sub(r"( )+", " ", value.strip())
     return dict_array
 
@@ -220,7 +228,7 @@ def has_double_colon(workbook_dict):
     for sheet in workbook_dict.values():
         for row in sheet:
             for column_header in row.keys():
-                if type(column_header) is not unicode:
+                if type(column_header) is not str:
                     continue
                 if "::" in column_header:
                     return True
@@ -270,9 +278,7 @@ def process_range_question_type(row):
     Raises PyXFormError when invalid range parameters are used.
     """
     new_dict = row.copy()
-    parameters = get_parameters(
-        new_dict.get("parameters", ""), ["start", "end", "step"]
-    )
+    parameters = get_parameters(new_dict.get("parameters", ""), ["start", "end", "step"])
     parameters_map = {"start": "start", "end": "end", "step": "step"}
     defaults = {"start": "1", "end": "10", "step": "1"}
 
@@ -298,13 +304,49 @@ def process_range_question_type(row):
     return new_dict
 
 
+def process_image_default(default_value):
+    # prepend image files with the correct prefix, if they don't have it.
+    image_jr_prefix = "jr://images/"
+    if image_jr_prefix not in default_value:
+        return "{}{}".format("jr://images/", default_value)
+    return default_value
+
+
+def find_sheet_misspellings(key: str, keys: "KeysView") -> "Optional[str]":
+    """
+    Find possible sheet name misspellings to warn the user about.
+
+    It's possible that this will warn about sheet names for sheets that have
+    auxilliary metadata that is not meant for processing by pyxform. For
+    example the "osm" sheet name may be similar to many other initialisms.
+
+    :param key: The sheet name to look for.
+    :param keys: The workbook sheet names.
+    """
+    candidates = tuple(
+        _k  # thanks to black
+        for _k in keys
+        if 2 >= levenshtein_distance(_k.lower(), key)
+        and _k not in constants.SUPPORTED_SHEET_NAMES
+        and not _k.startswith("_")
+    )
+    if 0 < len(candidates):
+        msg = (
+            "When looking for a sheet named '{k}', the following sheets with "
+            "similar names were found: {c}."
+        ).format(k=key, c=str(", ".join(("'{}'".format(c) for c in candidates))))
+        return msg
+    else:
+        return None
+
+
 def workbook_to_json(
     workbook_dict,
     form_name=None,
     fallback_form_name=None,
     default_language="default",
     warnings=None,
-):
+) -> "Dict[str, Any]":
     """
     workbook_dict -- nested dictionaries representing a spreadsheet.
                     should be similar to those returned by xls_to_dict
@@ -323,25 +365,35 @@ def workbook_to_json(
     returns a nested dictionary equivalent to the format specified in the
     json form spec.
     """
-    # ensure required headers are present
     if warnings is None:
         warnings = []
     is_valid = False
+    # Sheet names should be case-insensitive
     workbook_dict = {x.lower(): y for x, y in workbook_dict.items()}
+    workbook_keys = workbook_dict.keys()
+    if constants.SURVEY not in workbook_dict:
+        msg = "You must have a sheet named '{k}'. ".format(k=constants.SURVEY)
+        similar = find_sheet_misspellings(key=constants.SURVEY, keys=workbook_keys)
+        if similar is not None:
+            msg += similar
+        raise PyXFormError(msg)
+
+    # ensure required headers are present
     for row in workbook_dict.get(constants.SURVEY, []):
         is_valid = "type" in [z.lower() for z in row]
         if is_valid:
             break
     if not is_valid:
+        # TODO - could we state what headers are missing?
         raise PyXFormError(
-            "The survey sheet is either empty or missing important " "column headers."
+            "The survey sheet is either empty or missing important column headers."
         )
 
     row_format_string = "[row : %s]"
 
     # Make sure the passed in vars are unicode
-    form_name = unicode(form_name)
-    default_language = unicode(default_language)
+    form_name = str(form_name)
+    default_language = str(default_language)
 
     # We check for double columns to determine whether to use them
     # or single colons to delimit grouped headers.
@@ -353,6 +405,11 @@ def workbook_to_json(
     # Break the spreadsheet dict into easier to access objects
     # (settings, choices, survey_sheet):
     # ########## Settings sheet ##########
+    k = constants.SETTINGS
+    if k not in workbook_dict:
+        similar = find_sheet_misspellings(key=k, keys=workbook_keys)
+        if similar is not None:
+            warnings.append(similar + _MSG_SUPPRESS_SPELLING)
     settings_sheet_headers = workbook_dict.get(constants.SETTINGS, [])
     try:
         if (
@@ -411,7 +468,6 @@ def workbook_to_json(
     json_dict.update(settings)
 
     # ########## External Choices sheet ##########
-
     external_choices_sheet = workbook_dict.get(constants.EXTERNAL_CHOICES, [])
     for choice_item in external_choices_sheet:
         replace_smart_quotes_in_dict(choice_item)
@@ -424,37 +480,13 @@ def workbook_to_json(
     )
 
     # ########## Choices sheet ##########
-    # Columns and "choices and columns" sheets are deprecated,
-    # but we combine them with the choices sheet for backwards-compatibility.
-    choices_and_columns_sheet = workbook_dict.get(constants.CHOICES_AND_COLUMNS, {})
-    choices_and_columns_sheet = dealias_and_group_headers(
-        choices_and_columns_sheet,
-        aliases.list_header,
-        use_double_colons,
-        default_language,
-    )
-
-    columns_sheet = workbook_dict.get(constants.COLUMNS, [])
-    columns_sheet = dealias_and_group_headers(
-        columns_sheet, aliases.list_header, use_double_colons, default_language
-    )
-
     choices_sheet = workbook_dict.get(constants.CHOICES, [])
     for choice_item in choices_sheet:
         replace_smart_quotes_in_dict(choice_item)
-
     choices_sheet = dealias_and_group_headers(
         choices_sheet, aliases.list_header, use_double_colons, default_language
     )
-    # ########## Cascading Select sheet ###########
-    cascading_choices = workbook_dict.get(constants.CASCADING_CHOICES, [])
-    if len(cascading_choices):
-        if "choices" in cascading_choices[0]:
-            choices_sheet = choices_sheet + cascading_choices[0]["choices"]
-
-    combined_lists = group_dictionaries_by_key(
-        choices_and_columns_sheet + choices_sheet + columns_sheet, constants.LIST_NAME
-    )
+    combined_lists = group_dictionaries_by_key(choices_sheet, constants.LIST_NAME)
 
     choices = combined_lists
     # Make sure all the options have the required properties:
@@ -524,10 +556,6 @@ def workbook_to_json(
                             )  # noqa
 
     # ########## Survey sheet ###########
-    if constants.SURVEY not in workbook_dict:
-        raise PyXFormError(
-            "You must have a sheet named (case-sensitive): " + constants.SURVEY
-        )
     survey_sheet = workbook_dict[constants.SURVEY]
     # Process the headers:
     clean_text_values_enabled = aliases.yes_no.get(
@@ -540,6 +568,7 @@ def workbook_to_json(
     )
     survey_sheet = dealias_types(survey_sheet)
 
+    # No spell check for OSM sheet (infrequently used, many spurious matches).
     osm_sheet = dealias_and_group_headers(
         workbook_dict.get(constants.OSM, []), aliases.list_header, True
     )
@@ -547,7 +576,6 @@ def workbook_to_json(
     # #################################
 
     # Parse the survey sheet while generating a survey in our json format:
-
     row_number = 1  # We start at 1 because the column header row is not
     #                 included in the survey sheet (presumably).
     # A stack is used to keep track of begin/end expressions
@@ -561,6 +589,7 @@ def workbook_to_json(
     # If a group has a table-list appearance flag
     # this will be set to the name of the list
     table_list = None
+
     # For efficiency we compile all the regular expressions
     # that will be used to parse types:
     end_control_regex = re.compile(
@@ -577,11 +606,6 @@ def workbook_to_json(
         + r")) (?P<list_name>\S+)"
         + "( (?P<specify_other>(or specify other|or_other|or other)))?$"
     )
-    cascading_regexp = re.compile(
-        r"^(?P<cascading_command>("
-        + "|".join(aliases.cascading.keys())
-        + r")) (?P<cascading_level>\S+)?$"
-    )
     osm_regexp = re.compile(
         r"(?P<osm_command>(" + "|".join(aliases.osm.keys()) + r")) (?P<list_name>\S+)"
     )
@@ -619,8 +643,6 @@ def workbook_to_json(
         question_type = row.get(constants.TYPE)
         question_name = row.get(constants.NAME)
 
-        question_default = row.get("default")
-
         if not question_type:
             # if name and label are also missing,
             # then its a comment row, and we skip it with warning
@@ -673,9 +695,7 @@ def workbook_to_json(
                     new_dict["bind"].update(
                         {
                             "odk:"
-                            + constants.TRACK_CHANGES: parameters[
-                                constants.TRACK_CHANGES
-                            ]
+                            + constants.TRACK_CHANGES: parameters[constants.TRACK_CHANGES]
                         }
                     )
 
@@ -704,9 +724,7 @@ def workbook_to_json(
                     new_dict["bind"].update(
                         {
                             "odk:"
-                            + constants.IDENTIFY_USER: parameters[
-                                constants.IDENTIFY_USER
-                            ]
+                            + constants.IDENTIFY_USER: parameters[constants.IDENTIFY_USER]
                         }
                     )
 
@@ -827,7 +845,7 @@ def workbook_to_json(
         # on the survey sheet
         settings_type = aliases.settings_header.get(question_type)
         if settings_type:
-            json_dict[settings_type] = unicode(row.get(constants.NAME))
+            json_dict[settings_type] = str(row.get(constants.NAME))
             continue
 
         # Try to parse question as a end control statement
@@ -864,38 +882,18 @@ def workbook_to_json(
                 raise PyXFormError(
                     row_format_string % row_number + " Question or group with no name."
                 )
-        question_name = unicode(row[constants.NAME])
+        question_name = str(row[constants.NAME])
         if not is_valid_xml_tag(question_name):
             if isinstance(question_name, bytes):
                 question_name = question_name.encode("utf-8")
             error_message = row_format_string % row_number
             error_message += " Invalid question name [" + question_name + "] "
-            error_message += (
-                "Names must begin with a letter, colon," + " or underscore."
-            )
+            error_message += "Names must begin with a letter, colon," + " or underscore."
             error_message += (
                 "Subsequent characters can include numbers," + " dashes, and periods."
             )
             raise PyXFormError(error_message)
 
-        if (
-            constants.LABEL not in row
-            and row.get(constants.MEDIA) is None
-            and question_type not in aliases.label_optional_types
-            and not row.get("bind", {}).get("calculate")
-            and not (
-                row.get("default")
-                and default_is_dynamic(row.get("default"), question_type)
-            )
-        ):
-            # TODO: Should there be a default label?
-            #      Not sure if we should throw warnings for groups...
-            #      Warnings can be ignored so I'm not too concerned
-            #      about false positives.
-            warnings.append(
-                row_format_string % row_number + " Question has no label: " + str(row)
-            )
-
         # Try to parse question as begin control statement
         # (i.e. begin loop/repeat/group):
         begin_control_parse = begin_control_regex.search(question_type)
@@ -910,6 +908,35 @@ def workbook_to_json(
                 control_type = aliases.control[parse_dict["type"]]
                 control_name = question_name
 
+                # Check if the control item has a label, if applicable.
+                # This label check used to apply to all items, but no longer is
+                # relevant for questions since label nodes are added by default.
+                # There isn't an easy and neat place to put this besides here.
+                # Could potentially be simplified for control item cases.
+                if (
+                    constants.LABEL not in row
+                    and row.get(constants.MEDIA) is None
+                    and question_type not in aliases.label_optional_types
+                    and not row.get("bind", {}).get("calculate")
+                    and not (
+                        row.get("default")
+                        and default_is_dynamic(row.get("default"), question_type)
+                    )
+                    and not (
+                        control_type is constants.GROUP
+                        and row.get("control", {}).get("appearance")
+                        == constants.FIELD_LIST
+                    )
+                ):
+                    # Row number, name, and type probably enough for user message.
+                    # Also means the error message text is stable for tests.
+                    msg_dict = {"name": row.get("name"), "type": row.get("type")}
+                    warnings.append(
+                        row_format_string % row_number
+                        + " %s has no label: " % control_type.capitalize()
+                        + str(msg_dict)
+                    )
+
                 new_json_dict = row.copy()
                 new_json_dict[constants.TYPE] = control_type
                 child_list = list()
@@ -932,9 +959,7 @@ def workbook_to_json(
 
                 # Generate a new node for the jr:count column so
                 # xpath expressions can be used.
-                repeat_count_expression = new_json_dict.get("control", {}).get(
-                    "jr:count"
-                )
+                repeat_count_expression = new_json_dict.get("control", {}).get("jr:count")
                 if repeat_count_expression:
                     generated_node_name = new_json_dict["name"] + "_count"
                     parent_children_array.append(
@@ -964,7 +989,7 @@ def workbook_to_json(
                         appearance_string = "field-list"
                         for w in appearance_mods_as_list:
                             if w != constants.TABLE_LIST:
-                                appearance_string += " " + unicode(w)
+                                appearance_string += " " + str(w)
                         new_json_dict["control"]["appearance"] = appearance_string
 
                         # Generate a note label element so hints and labels
@@ -976,9 +1001,9 @@ def workbook_to_json(
                                 "name": "generated_table_list_label_" + str(row_number),
                             }
                             if "label" in new_json_dict:
-                                generated_label_element[
+                                generated_label_element[constants.LABEL] = new_json_dict[
                                     constants.LABEL
-                                ] = new_json_dict[constants.LABEL]
+                                ]
                                 del new_json_dict[constants.LABEL]
                             if "hint" in new_json_dict:
                                 generated_label_element["hint"] = new_json_dict["hint"]
@@ -998,54 +1023,6 @@ def workbook_to_json(
                 )
                 continue
 
-        # try to parse as a cascading select
-        cascading_parse = cascading_regexp.search(question_type)
-        if cascading_parse:
-            parse_dict = cascading_parse.groupdict()
-            if parse_dict.get("cascading_command"):
-                cascading_level = parse_dict["cascading_level"]
-                cascading_prefix = row.get(constants.NAME)
-                if not cascading_prefix:
-                    raise PyXFormError(
-                        row_format_string % row_number
-                        + " Cascading select needs a name."
-                    )
-                # cascading_json = get_cascading_json(
-                # cascading_choices, cascading_prefix, cascading_level)
-                if (
-                    len(cascading_choices) <= 0
-                    or "questions" not in cascading_choices[0]
-                ):
-                    raise PyXFormError(
-                        "Found a cascading_select "
-                        + cascading_level
-                        + ", but could not find "
-                        + cascading_level
-                        + "in cascades sheet."
-                    )
-                cascading_json = cascading_choices[0]["questions"]
-                json_dict["choices"] = choices
-                include_bindings = False
-                if "bind" in row:
-                    include_bindings = True
-                for cq in cascading_json:
-                    # include bindings
-                    if include_bindings:
-                        cq["bind"] = row["bind"]
-
-                    def replace_prefix(d, prefix):
-                        for k, v in d.items():
-                            if isinstance(v, basestring):
-                                d[k] = v.replace("$PREFIX$", prefix)
-                            elif isinstance(v, dict):
-                                d[k] = replace_prefix(v, prefix)
-                            elif isinstance(v, list):
-                                d[k] = map(lambda x: replace_prefix(x, prefix), v)
-                        return d
-
-                    parent_children_array.append(replace_prefix(cq, cascading_prefix))
-                continue  # so the row isn't put in as is
-
         # Try to parse question as a select:
         select_parse = select_regexp.search(question_type)
         if select_parse:
@@ -1064,12 +1041,22 @@ def workbook_to_json(
                     select_type == "select one external"
                     and list_name not in external_choices
                 ):
+                    if not external_choices:
+                        k = constants.EXTERNAL_CHOICES
+                        msg = "There should be an external_choices sheet in this xlsform."
+                        similar = find_sheet_misspellings(key=k, keys=workbook_keys)
+                        if similar is not None:
+                            msg = msg + " " + similar
+                        raise PyXFormError(
+                            msg
+                            + " Please ensure that the external_choices sheet has columns"
+                            " 'list name', and 'name'."
+                        )
                     raise PyXFormError(
                         row_format_string % row_number
                         + "List name not in external choices sheet: "
                         + list_name
                     )
-                    select_type = aliases.select["select_one"]
                 if (
                     list_name not in choices
                     and select_type != "select one external"
@@ -1077,11 +1064,14 @@ def workbook_to_json(
                     and not re.match(r"\$\{(.*?)\}", list_name)
                 ):
                     if not choices:
+                        k = constants.CHOICES
+                        msg = "There should be a choices sheet in this xlsform."
+                        similar = find_sheet_misspellings(key=k, keys=workbook_keys)
+                        if similar is not None:
+                            msg = msg + " " + similar
                         raise PyXFormError(
-                            "There should be a choices sheet in this xlsform."
-                            " Please ensure that the choices sheet name is "
-                            "all in small caps and has columns 'list name', "
-                            "'name', and 'label' (or aliased column names)."
+                            msg + " Please ensure that the choices sheet has the"
+                            " mandatory columns 'list_name', 'name', and 'label'."
                         )
                     raise PyXFormError(
                         row_format_string % row_number
@@ -1109,31 +1099,6 @@ def workbook_to_json(
                 specify_other_question = None
                 if parse_dict.get("specify_other") is not None:
                     select_type += " or specify other"
-                    # With this code we no longer need to handle or_other
-                    # questions in survey builder.
-                    # However, it depends on being able to use choice filters
-                    # and xpath expressions that return empty sets.
-                    # choices[list_name].append(
-                    # {
-                    #     'name': 'other',
-                    #     'label': {default_language : 'Other'},
-                    #     'orOther': 'true',
-                    # })
-                    # or_other_xpath = 'isNull(orOther)'
-                    # if 'choice_filter' in row:
-                    #   row['choice_filter'] += ' or ' + or_other_xpath
-                    # else:
-                    #   row['choice_filter'] = or_other_xpath
-
-                    # specify_other_question = \
-                    # {
-                    #       'type':'text',
-                    #       'name': row['name'] + '_specify_other',
-                    #       'label':
-                    #        'Specify Other for:\n"' + row['label'] + '"',
-                    #       'bind' : {'relevant':
-                    #                "selected(../%s, 'other')" % row['name']},
-                    #     }
 
                 new_json_dict = row.copy()
                 new_json_dict[constants.TYPE] = select_type
@@ -1178,8 +1143,7 @@ def workbook_to_json(
                             new_json_dict["list_name"] = list_name
                             new_json_dict[constants.CHOICES] = choices[list_name]
                 elif (
-                    "randomize" in parameters.keys()
-                    and parameters["randomize"] == "true"
+                    "randomize" in parameters.keys() and parameters["randomize"] == "true"
                 ):
                     new_json_dict["itemset"] = list_name
                     json_dict["choices"] = choices
@@ -1195,7 +1159,7 @@ def workbook_to_json(
                 # (for groups of selects)
                 if table_list is not None:
                     # Then this row is the first select in a table list
-                    if not isinstance(table_list, basestring):
+                    if not isinstance(table_list, str):
                         table_list = list_name
                         table_list_header = {
                             constants.TYPE: select_type,
@@ -1213,16 +1177,11 @@ def workbook_to_json(
                         error_message = row_format_string % row_number
                         error_message += (
                             " Badly formatted table list,"
-                            " list names don't match: "
-                            + table_list
-                            + " vs. "
-                            + list_name
+                            " list names don't match: " + table_list + " vs. " + list_name
                         )
                         raise PyXFormError(error_message)
 
-                    control = new_json_dict["control"] = new_json_dict.get(
-                        "control", {}
-                    )
+                    control = new_json_dict["control"] = new_json_dict.get("control", {})
                     control["appearance"] = "list-nolabel"
                 parent_children_array.append(new_json_dict)
                 if specify_other_question:
@@ -1255,15 +1214,16 @@ def workbook_to_json(
 
         if question_type == "photo":
             new_dict = row.copy()
-            parameters = get_parameters(row.get("parameters", ""), ["max-pixels"])
 
+            if row.get("default"):
+                new_dict["default"] = process_image_default(row["default"])
+            # Validate max-pixels
+            parameters = get_parameters(row.get("parameters", ""), ["max-pixels"])
             if "max-pixels" in parameters.keys():
                 try:
                     int(parameters["max-pixels"])
                 except ValueError:
-                    raise PyXFormError(
-                        "Parameter max-pixels must have an integer value."
-                    )
+                    raise PyXFormError("Parameter max-pixels must have an integer value.")
 
                 new_dict["bind"] = new_dict.get("bind", {})
                 new_dict["bind"].update({"orx:max-pixels": parameters["max-pixels"]})
@@ -1312,6 +1272,24 @@ def workbook_to_json(
             parent_children_array.append(new_dict)
             continue
 
+        if question_type in ["geopoint", "geoshape", "geotrace"]:
+            new_dict = row.copy()
+            parameters = get_parameters(
+                row.get("parameters", ""), ["allow-mock-accuracy"]
+            )
+
+            if "allow-mock-accuracy" in parameters.keys():
+                if parameters["allow-mock-accuracy"] not in ["true", "false"]:
+                    raise PyXFormError("Invalid value for allow-mock-accuracy.")
+
+                new_dict["bind"] = new_dict.get("bind", {})
+                new_dict["bind"].update(
+                    {"odk:allow-mock-accuracy": parameters["allow-mock-accuracy"]}
+                )
+
+            parent_children_array.append(new_dict)
+            continue
+
         # TODO: Consider adding some question_type validation here.
 
         # Put the row in the json dict as is:
@@ -1412,7 +1390,7 @@ def parse_file_to_json(
     if warnings is None:
         warnings = []
     workbook_dict = parse_file_to_workbook_dict(path, file_object)
-    fallback_form_name = unicode(get_filename(path))
+    fallback_form_name = str(get_filename(path))
     return workbook_to_json(
         workbook_dict, default_name, fallback_form_name, default_language, warnings
     )
@@ -1466,7 +1444,7 @@ def get_parameters(raw_parameters, allowed_parameters):
     return params
 
 
-class SpreadsheetReader(object):
+class SpreadsheetReader:
     def __init__(self, path_or_file):
         path = path_or_file
         try:
@@ -1475,7 +1453,7 @@ class SpreadsheetReader(object):
             pass
         self._dict = parse_file_to_workbook_dict(path)
         self._path = path
-        self._id = unicode(get_filename(path))
+        self._id = str(get_filename(path))
         self._name = self._print_name = self._title = self._id
 
     def to_json_dict(self):
@@ -1496,7 +1474,7 @@ class SurveyReader(SpreadsheetReader):
     """
 
     def __init__(self, path_or_file, default_name=None):
-        if isinstance(path_or_file, basestring):
+        if isinstance(path_or_file, str):
             self._file_object = None
             path = path_or_file
         else:
@@ -1553,9 +1531,7 @@ class VariableNameReader(SpreadsheetReader):
         assert "Dictionary" in self._dict
         for d in self._dict["Dictionary"]:
             if "Variable Name" in d:
-                assert d["Variable Name"] not in variable_names_so_far, d[
-                    "Variable Name"
-                ]
+                assert d["Variable Name"] not in variable_names_so_far, d["Variable Name"]
                 variable_names_so_far.append(d["Variable Name"])
                 new_dict[d["XPath"]] = d["Variable Name"]
             else:
@@ -1568,17 +1544,17 @@ if __name__ == "__main__":
     # convert that file to json, then print it
     if len(sys.argv) < 2:
         # print "You must supply a file argument."
-        filename = "xlsform_spec_test.xls"
-        path = "/home/user/python-dev/xlsform/pyxform/tests/example_xls/"
-        path += filename
+        _filename = "xlsform_spec_test.xls"
+        _path = "/home/user/python-dev/xlsform/pyxform/tests/example_xls/"
+        _path += _filename
     else:
-        path = sys.argv[1]
+        _path = sys.argv[1]
 
-    warnings = []
+    _warnings = []
 
-    json_dict = parse_file_to_json(path, warnings=warnings)
-    print_pyobj_to_json(json_dict)
+    _json_dict = parse_file_to_json(path=_path, warnings=_warnings)
+    print_pyobj_to_json(pyobj=_json_dict)
 
-    if len(warnings) > 0:
+    if len(_warnings) > 0:
         sys.stderr.write("Warnings:" + "\n")
-        sys.stderr.write("\n".join(warnings) + "\n")
+        sys.stderr.write("\n".join(_warnings) + "\n")
